package jasper.data;
import jasper.element.axioms.*;
import java.util.*;
import jasper.element.*;
import jasper.element.symbols.*;
import jasper.element.operators.*;
import jasper.main.*;
import jasper.exceptions.*;
//Axiomatic Translation, version 1.1.0, @K.J.Smith 2007-2008, http://project.kjsmith.net/
public class OutputTerm implements Constants {

    private final OutputTerm.Axioms axioms = new OutputTerm.Axioms();
    private final OutputTerm.InputTerm inputTerm = new OutputTerm.InputTerm();
    private final InstantiationSet initialISet = new InstantiationSet();
    private final ArrayList initialDefs = new ArrayList();
    private final ArrayList andXtraDefs = new ArrayList();
    private final TranslationManager tm = TranslationManager.getTransationManager();
    private int INITIAL = 0;
    private int ANDXTRA = 1;
    private boolean translated = false;
 
    public OutputTerm() { 
    }

    public void translate() {
        if (this.isInitialized()==false) {
            throw new JasperRuntimeException();
        }
        this.initializeInstantiationSet();
        this.translateInputTerm();
        axioms.translate();
        this.translated = true;
    }
    
    private void initializeInstantiationSet () {
        this.initialISet.buildISet(this.inputTerm.getTerm(), ModalityIndex.dummy);
        //System.out.println("Initial instantiationSet = " + this.initialISet.toString());
    }
    
    private void updateDefsList (int mode, Def def) {
        ArrayList working = null;
        if (mode==INITIAL) {
            working = this.initialDefs;
        } else if (mode==ANDXTRA) {
            working = this.andXtraDefs;
        } else {
            throw new JasperRuntimeException();
        }
        Iterator it = working.iterator();
        boolean addTo = true;
        while (it.hasNext()) {
            Def tmp = (Def)it.next();
            if (def.equals(tmp)) {
                addTo = false;
            }
        }    
        if (addTo == true) {
            working.add(def);
        }
    }
    
    private void translateInputTerm () {
        Iterator it = this.initialISet.getISet().iterator();
        while (it.hasNext()) {
            Def def = new Def(((Term)it.next()).copy());
            //System.out.println("def term = "+def.getTerm().toString());
            this.updateDefsList(INITIAL, def);
        }
        andXtraDefs.addAll(initialDefs);
    }
    
    private InstantiationSet getInstantiationSet () {
        return this.initialISet;
    }
    
    private Exists buildETerm () { //ExQp(x)
        Predicate symbol = inputTerm.getTerm().copy().getP();
        FOLVariable X = tm.getSymbolCache().getVariable(VarX);
        symbol.setArg(X);
        return new Exists(symbol, new QuantifierList(X));
    }
    
    public Term getOutTerm() {
        if (this.translated == false) {
            throw new JasperRuntimeException();
        }
        And and = null;
        FOLVariable X = tm.getSymbolCache().getVariable(VarX);
        
        //build E term
        and = new And (this.buildETerm());
            
        //process defs
        Iterator it = andXtraDefs.iterator();
        while (it.hasNext()) {
            Def tmp = (Def)it.next();
            and.setTerm(tmp.getTerm().copy());
        }
        
        //process axioms -- defs already been dealt with above
        if ((axioms.getConcatenatedResults()!=null)
                &&(axioms.getConcatenatedResults().getTerm()!=null)) {
            and.setTerm(axioms.getConcatenatedResults().getTerm());
        }
        
        return and;
    }
    
    public boolean isInitialized() {
        return this.inputTerm.isInitialized();
    }

    public void setTerm(Term term) {
        this.inputTerm.setTerm(term);
    }

    public void addAxiom(Axiom axiom) {
        this.axioms.addAxiom(axiom);
    }
    
    //update global 'XtraDefs'
    private void update (Def[] defs) {
        if (defs==null) {
            return;
        }
        for (int i = 0; i < defs.length; i++) {
            this.updateDefsList (ANDXTRA, defs[i]);
        }
    }
    
    //------------------------------------------------------
    private static class InputTerm {

        private Term term = null;

        private InputTerm() {
        }

        private void setTerm(Term term) {
            if (this.isInitialized() == false) {
                this.term = term;
            } else {
                throw new JasperRuntimeException();
            }
        }

        private Term getTerm() {
            return this.term.copy();
        }

        private boolean isInitialized() {
            if (term == null) {
                return false;
            } else {
                return true;
            }
        }
    }

    //------------------------------------------------------
    private class Axioms extends ArrayList {

        private Iterator it = this.iterator();
        private And out = null;
        
        private Axioms() {
        }

        private void addAxiom(Axiom axiom) {
            this.add(axiom);
        }

        private Axiom getNextAxiom() {
            if (this.it.hasNext()) {
                return (Axiom)it.next();
            } else {
                return null;
            }
        }

        private boolean hasNextAxiom() {
            return it.hasNext();
        }
        
        private void translate() {
            InstantiationSet is = OutputTerm.this.getInstantiationSet().copy();
            Iterator it = this.iterator();
            Def[] delayedDefs = null;
            while (it.hasNext()) {
                OutputTerm.this.update(delayedDefs);
                Axiom ax = (Axiom) it.next();
                ax.setInstantiationSet(is.copy());
                OutputTerm.this.update(ax.getDef());
                
//                Def[] tmp = ax.getDef();
//                if (tmp!=null) {
//                for (int i=0; i < tmp.length;i++) {
//                    System.out.println("*************"+tmp[i].getFormingTerm().toString());
//                }
//                } else { System.out.println("DEF WAS NULL");}
                
                delayedDefs=ax.getCompositionalDef();
                Term setme = ax.getResult();
                if (setme!=null) {
                    if (this.out==null) {
                        this.out = new And(setme);
                    } else {                    
                        this.out.setTerm(setme);
                    }
                }
                is = is.plus(ax.getCompositionalISet());
            }
        }
        
        private And getConcatenatedResults () {
            return this.out;
        }
        
        private void reset () {
            this.it = this.iterator();
        }
        
        private boolean isInitialized() {
            if (this.isEmpty() == true) {
                return false;
            } else {
                return true;
            }
        }
    }
    //    public static void main(String[] args) {
    //        ;
    //    }
}
