package jasper.element;
import jasper.exceptions.*;
import java.util.*;
import jasper.element.symbols.*;
import jasper.element.interfaceAbstract.*;
//Axiomatic Translation, version 1.1.0, @K.J.Smith 2007-2008, http://project.kjsmith.net/
public abstract class ModalTerm extends Term implements Binary {

    private final Term term;
    private final ModalityIndex index;

    public ModalTerm(ModalityIndex r, Term term, String name)
            throws TermCompositionException {
        super(name);
        if ((term==null)||(r==null)) { 
            throw new JasperRuntimeException();
        }
        this.term = term;
        this.index = r;
        if (this.checkTypeOK() == false) {
            throw new TermCompositionException();
        }
        this.setNemonic(this.nemonic);
    }

    public Term getTerm() {
        return this.term.copy();
    }

    public boolean equals(Term term) {
        if (term==null) {
            throw new JasperRuntimeException();
        }
        if ((term instanceof ModalTerm) 
                && (this.getIndex().equals(((ModalTerm) term).getIndex())==true) 
                && (this.getTerm().equals(((ModalTerm) term).getTerm())==true) ) {
            return true;
        } else {
            return false;
        }
    }
    
    public ModalityIndex getIndex() {
        return this.index.copy();
    }

    public ArrayList descendList() {
        ArrayList out = new ArrayList();
        out.add(this);
        out.addAll(this.getTerm().descendList());
        return out;
    }
    
    protected final boolean checkTypeOK() {
        if (this.term instanceof QuantifierTerm) {
            return false;
        } else if (this.term instanceof Quantifier) {
            return false;
        } else if (this.term instanceof TermElement) {
            return false;
        } 
        return true;
    }

    public String toString() {
        return this.getName() 
                + "(" + this.index.toString() 
                + "," + this.term.toString() + ")";
    }
    
    public String asString() {
        return this.getName() 
                + "_" + this.index.asString() 
                + "_" + this.term.asString();// + "_";
    }
    
    public String inString() {
        return this.getNemonic() 
                + "(" + this.index.inString() 
                + "," + this.term.inString() + ")";
    }
    
//    public static void main(String[] args) {
//        ;
//    }
}
