package jasper.element;
import jasper.element.operators.*;
import jasper.exceptions.*;
import java.util.*;
import jasper.element.interfaceAbstract.*;
//Axiomatic Translation, version 1.1.0, @K.J.Smith 2007-2008, http://project.kjsmith.net/
public abstract class NaryTerm extends Term implements Nary {

    private final ArrayList termList = new ArrayList();

    public NaryTerm(Term term, String name)
            throws TermCompositionException {
        super(name);
        this.setTerm(term);
        if (this.checkTypeOK() == false) {
            throw new TermCompositionException();
        }
        this.setNemonic(this.nemonic);
    }

    public NaryTerm(Term term1, Term term2, String name)
            throws TermCompositionException {
        super(name);
        this.setTerm(term1);
        this.setTerm(term2);
        if (this.checkTypeOK() == false) {
            throw new TermCompositionException();
        }
        this.setNemonic(this.nemonic);
    }

    public NaryTerm(Term term1, Term term2, Term term3, String name)
            throws TermCompositionException {
        super(name);
        this.setTerm(term1);
        this.setTerm(term2);
        this.setTerm(term3);
        if (this.checkTypeOK() == false) {
            throw new TermCompositionException();
        }
        this.setNemonic(this.nemonic);
    }
    
    public NaryTerm(Term term1, Term term2, Term term3, Term term4, String name)
            throws TermCompositionException {
        super(name);
        this.setTerm(term1);
        this.setTerm(term2);
        this.setTerm(term3);
        this.setTerm(term4);
        if (this.checkTypeOK() == false) {
            throw new TermCompositionException();
        }
        this.setNemonic(this.nemonic);
    }
    
    public NaryTerm(ArrayList list, String name)
            throws TermCompositionException {
        super(name);
        this.setTerm(list);
        if (this.checkTypeOK() == false) {
            throw new TermCompositionException();
        }
        this.setNemonic(this.nemonic);
    }

    public final void setTerm(Term term)
            throws TermCompositionException {
        if (term==null) { 
            throw new JasperRuntimeException();
        }
        this.termList.add(term);
    }

    public final void setTerm(Term term1, Term term2)
            throws TermCompositionException {
        this.setTerm(term1);
        this.setTerm(term2);
    }

    public final void setTerm(ArrayList list)
            throws TermCompositionException {
        if (list==null) { 
            throw new JasperRuntimeException();
        }
        Iterator it = list.iterator();
        while (it.hasNext()) {
            Term tmp = (Term)it.next();
            if ( (tmp==null) 
                || ((tmp instanceof Term)==false)) {
                throw new TermCompositionException();
            }
        }
        this.termList.addAll(list);
    }

    public final ArrayList getTerm() {
        ArrayList out = new ArrayList();
        Iterator it = this.termList.iterator();
        while (it.hasNext()) {
            out.add(((Term)it.next()).copy());
        }
        return out;
    }

    public boolean equals (Term term) {
        if (term==null) {
            throw new JasperRuntimeException();
        }
        if ((term instanceof NaryTerm)==false) {
            return false;
        }
        Iterator it = this.getTerm().iterator();
        while (it.hasNext()) {
            Term tmp = (Term)it.next();
            Iterator it2 = ((NaryTerm)term).getTerm().iterator();
            boolean found = false;
            while (it2.hasNext()) { 
                if (tmp.equals((Term)it2.next())==true) {
                    found = true;
                }
            }
            if (found==false) {
                return false;
            }
        }
        //now other way around
        it = ((NaryTerm)term).getTerm().iterator();
        while (it.hasNext()) {
            Term tmp = (Term)it.next();
            Iterator it2 = this.getTerm().iterator();
            boolean found = false;
            while (it2.hasNext()) { 
                if (tmp.equals((Term)it2.next())==true) {
                    found = true;
                }
            }
            if (found==false) {
                return false;
            }
        }
        
        return true;
    }
    
    public ArrayList descendList() {
        ArrayList out = new ArrayList();
        out.add(this);
        Iterator it = this.getTerm().iterator();
        while (it.hasNext()) {
            out.addAll(((Term) it.next()).descendList());
        }
        return out;
    }
   
    protected boolean checkTypeOK() {
        Iterator it = this.getTerm().iterator();
        while (it.hasNext()) {
            Term tmp = (Term)it.next();
            if (tmp instanceof TermElement) {
                return false;
            } else if (tmp instanceof QuantifierList) {
                return false;
            }
        }
        return true;
    }

    public String toString() {
        String str = this.getName() + "(";
        Iterator it = this.termList.iterator();
        while (it.hasNext()) {
            str += ((Term) it.next()).toString();
            if (it.hasNext()) {
                str += ",";
            }
        }
        str += ")";
        return str;
    }
    
    public String inString() {
        String str = this.getNemonic() + "(";
        Iterator it = this.termList.iterator();
        while (it.hasNext()) {
            str += ((Term) it.next()).inString();
            if (it.hasNext()) {
                str += ",";
            }
        }
        str += ")";
        return str;
    }
    
    public String asStringOr() {
        String str = this.getName() + "_";
        Iterator it = this.termList.iterator();
        while (it.hasNext()) {
            str += ((Term) it.next()).asString();
            if (it.hasNext()) {
                str += "_";
            }
        }
        return str;
    }
    
    public String asString() {
        String str = this.getName() + "_";
        String[] sorted = this.sortedAndOrString();
        for (int i = 0; i < sorted.length; i++) {
            str += sorted[i];
            if ((i + 1) < sorted.length) {
                str += "_";
            }
        }
//        Iterator it = this.termList.iterator();
//        while (it.hasNext()) {
//            str += ((Term) it.next()).asString();
//            if (it.hasNext()) {
//                str += "_";
//            }
//        }
        return str;
    }
    
    private String[] sortedAndOrString () {
        ArrayList list = new ArrayList();
        Iterator it = this.termList.iterator();
        while (it.hasNext()) {
            list.add(((Term) it.next()).asString());
            
        }
        Comparator comparator = new Comparator() 
        {  
            public int compare(Object o1, Object o2) 
            {  
                String str1 = (String)o1;
                String str2 = (String)o2;
                return str1.compareTo(str2); 
            } 
        }; 
        Collections.sort(list, comparator);
        return (String[])list.toArray(new String[list.size()]);
    }
    
//    public static void main(String[] args) {
//        ;
//    }
}
