package jasper.element.axioms;
import jasper.element.symbols.*;
import jasper.element.*;
import java.util.*;
import jasper.data.*;
import jasper.element.operators.*;
//Axiomatic Translation, version 1.1.0, @K.J.Smith 2007-2008, http://project.kjsmith.net/
public class Axiom4 extends Axiom {
    
    private final String name = "Axiom4";
    //Transitive  []p->[][]p   Ax(¬Q[]p(x)\/Ay(¬R(x,y)\/Q[]p(y)) )
    public Axiom4 (ModalityIndex index) {
        super(index);
        this.setModeComposition(true);
    } 
    
    //Ax( ¬Q[]p(x) \/ Ay(¬R(x,y)\/Q[]p(y)) )
    protected final Term getResult(InstantiationSet iset) {
        And returnedTerm = null;
        SymbolCache sc = this.getTM().getSymbolCache();
        FOLVariable X = sc.getVariable(VarX);
        FOLVariable Y = sc.getVariable(VarY);
        Iterator it = iset.getBoxISet().iterator();
        
        while (it.hasNext()) {
            Term out = null;
            Box def = ((Box) it.next()).copy();
            Predicate symbolBoxP = def.copy().getP(); //Q[]p
            AccessibilityRelation Rxy = sc.getAccessibilityRelation(def.getIndex().copy());
            ModalityIndex r = def.getIndex().copy();
            Rxy.setVariables(X, Y, r); //Rxy
            Term RxyTerm = Rxy.negate();

            Predicate first = symbolBoxP.copy();
            first.setArg(X);
            Term firstTerm = first.negate(); //¬Q[]p(x)
            
            Predicate second = symbolBoxP.copy();
            second.setArg(Y); //Q[]p(y)
            
            out = new Forall(new Or(second, RxyTerm), new QuantifierList(Y));
            out = new Forall(new Or(out, firstTerm), new QuantifierList(X));
            if (returnedTerm==null) {
                returnedTerm = new And(out);
            } else {
                returnedTerm.setTerm(out);
            }
        }
        return returnedTerm;
    }

    public Def[] getDef() { //no new terms 
        return null;
    }
    
    protected InstantiationSet getISet() {  //[][]p
        InstantiationSet is = new InstantiationSet();
        if ((this.getInstantiationSet()==null) 
                ||(this.getInstantiationSet().getBoxISet()==null)) {
            return null;
        }
        Iterator it = this.getInstantiationSet().getBoxISet().iterator();
        while (it.hasNext()) {
            Term in = ((Term) it.next()).copy();
            ModalityIndex r = ((Box) in).getIndex();
            is.buildISet(in.box(r.copy()), this.getModalityIndex());
        }
        return is;
    }

    public final String getName() {
        String out = this.name;
        if (this.isModeComposition()==true) {
            out += "o";
        }
        return out;
    }
    
//    public static void main(String[] args) {
//        ;
//    }
}
