package jasper.element.axioms;
import jasper.element.symbols.*;
import jasper.element.*;
import java.util.*;
import jasper.data.*;
import jasper.element.operators.*;
//Axiomatic Translation, version 1.1.0, @K.J.Smith 2007-2008, http://project.kjsmith.net/
public class AxiomB extends Axiom {
    
    private final String name = "AxiomB";
    //Symmetry  ¬[]¬[]p->p     Ax( Ay(¬R(x,y)\/¬Q[]p(y))\/Qp(x) )
    public AxiomB (ModalityIndex index) {
        super(index);
        this.setModeComposition(true);
    }
    
    //Ax( Ay(¬R(x,y)\/¬Q[]p(y)) \/ Qp(x) )
    protected final Term getResult(InstantiationSet iset) {
        And returnedTerm = null;
        SymbolCache sc = this.getTM().getSymbolCache();
        FOLVariable X = sc.getVariable(VarX);
        FOLVariable Y = sc.getVariable(VarY);
        Iterator it = iset.getBoxISet().iterator();
        
        while (it.hasNext()) {
            Term out = null;
            Box def = ((Box) it.next()).copy();
            Predicate symbolBoxP = def.copy().getP(); //Q[]p
            Predicate symbolP = def.getTerm().copy().getP(); //Qp
            AccessibilityRelation Rxy = sc.getAccessibilityRelation(def.getIndex());
            ModalityIndex r = def.getIndex().copy();
            Rxy.setVariables(X, Y, r); //Rxy
            Term RxyTerm = Rxy.negate();

            symbolP.setArg(X); //Qp(x)
            symbolBoxP.setArg(Y); //Q[]p(y)
            Term symbolBoxPTerm = symbolBoxP.negate(); //¬Q[]p(y)
            
            out = new Forall(new Or(symbolBoxPTerm, RxyTerm), new QuantifierList(Y));
            out = new Forall(new Or(symbolP, out), new QuantifierList(X));
            if (returnedTerm==null) {
                returnedTerm = new And(out);
            } else {
                returnedTerm.setTerm(out);
            }
        }
        return returnedTerm;
    }

    public Def[] getDef() { //no new terms
        return null;
    }
        
    protected InstantiationSet getISet() { //[]¬[]p
        InstantiationSet is = new InstantiationSet();
        Iterator it = this.getInstantiationSet().getBoxISet().iterator();

        while (it.hasNext()) {
            Term in = ((Term) it.next()).copy();
            ModalityIndex r = ((Box) in).getIndex().copy();
            is.buildISet(in.negate().box(r.copy()), this.getModalityIndex());
        }
        return is;
    }

    public final String getName() {
        String out = this.name;
        if (this.isModeComposition()==true) {
            out += "o";
        }
        return out;
    }
//    public static void main(String[] args) {
//        ;
//    }
}
