package jasper.element.axioms;
import jasper.element.symbols.*;
import jasper.element.*;
import java.util.*;
import jasper.data.*;
import jasper.element.operators.*;
//Axiomatic Translation, version 1.1.0, @K.J.Smith 2007-2008, http://project.kjsmith.net/
public class AxiomB3 extends Axiom {
    
    private final String name = "AxiomB3";

    public AxiomB3(ModalityIndex index) {
        super(index);
        this.setModeComposition(true);
        this.setModeCf5(true);
    }

    //Ax(Ay(¬R(x,y)  v Az(¬R(y,z) v Au(¬R(z,u) v  ¬Q[]p(u)))) v Qp(x))
    protected final Term getResult(InstantiationSet iset) {
        And returnedTerm = null;
        SymbolCache sc = this.getTM().getSymbolCache();
        FOLVariable X = sc.getVariable(VarX);
        FOLVariable Y = sc.getVariable(VarY);
        FOLVariable Z = sc.getVariable(VarZ);
        FOLVariable U = sc.getVariable(VarU);
        Iterator it = iset.getBoxISet().iterator();
        
        while (it.hasNext()) {
            Term out = null;
            Box def = ((Box) it.next()).copy();
            Predicate symbolBoxP = def.copy().getP(); //Q[]p
            Predicate symbolP = def.getTerm().copy().getP(); //Qp
            
            AccessibilityRelation Rxy = sc.getAccessibilityRelation(def.getIndex().copy());
            ModalityIndex r = def.getIndex().copy();
            Rxy.setVariables(X, Y, r); 
            Term RxyTerm = Rxy.negate();

            AccessibilityRelation Ryz = sc.getAccessibilityRelation(def.getIndex().copy());
            r = def.getIndex().copy();
            Ryz.setVariables(Y, Z, r); 
            Term RyzTerm = Ryz.negate();
            
            AccessibilityRelation Rzu = sc.getAccessibilityRelation(def.getIndex().copy());
            r = def.getIndex().copy();
            Rzu.setVariables(Z, U, r); 
            Term RzuTerm = Rzu.negate();
            //Ax(Ay(¬R(x,y)  v Az(¬R(y,z) v Au(¬R(z,u) v  ¬Q[]p(u)))) v Qp(x))
            Predicate first = symbolP.copy();
            first.setArg(X);  
            
            Predicate second = symbolBoxP.copy();
            second.setArg(U);
            Term secondTerm = second.negate(); 
            
            out = new Forall(new Or(secondTerm, RzuTerm), new QuantifierList(U));
            out = new Forall(new Or(out, RyzTerm), new QuantifierList(Z));
            out = new Forall(new Or(out, RxyTerm), new QuantifierList(Y));
            out = new Forall(new Or(first, out), new QuantifierList(X));
            if (returnedTerm==null) {
                returnedTerm = new And(out);
            } else {
                returnedTerm.setTerm(out);
            }
        }
        return returnedTerm;
    }

    public Def[] getDef() { 
        if (this.isModeCf5()==true) {
            ArrayList defs = new ArrayList();
            if ((this.getISet()==null) 
                || (this.getISet().getBoxISet()==null)) {
                return null;
            }
            Iterator it = this.getISet().getBoxISet().iterator();
            while (it.hasNext()) {
                defs.add(new Def(((Term) it.next()).copy()));
            }
            return (Def[]) defs.toArray(new Def[defs.size()]);
        } else { //no new def terms
            return null;
        }
    }

    protected InstantiationSet getISet() {  //[]¬[]p, [][]¬[]p, [][][]¬[]p
        //System.out.println("XXXXXXXXXXXXXXXXXXXXXX");
        InstantiationSet is = new InstantiationSet();
        if ((this.getInstantiationSet() == null) 
                || (this.getInstantiationSet().getBoxISet() == null)) {
            return null;
        }
        Iterator it = this.getInstantiationSet().getBoxISet().iterator();
        while (it.hasNext()) {
            Box in = ((Box) it.next()).copy();
            ModalityIndex r = in.getIndex();
            is.buildISet(in.negate().box(r.copy()), this.getModalityIndex());
        }
        it = this.getInstantiationSet().getBoxISet().iterator();
        while (it.hasNext()) {
            Box in = ((Box) it.next()).copy();
            ModalityIndex r = in.getIndex();
            is.buildISet(in.negate().box(r.copy()).box(r.copy()), this.getModalityIndex());
        }
        it = this.getInstantiationSet().getBoxISet().iterator();
        while (it.hasNext()) {
            Box in = ((Box) it.next()).copy();
            ModalityIndex r = in.getIndex();
            is.buildISet(in.negate().box(r.copy()).box(r.copy()).box(r.copy()), this.getModalityIndex());
        }
        return is;
    }
    
    public final String getName() {
        String out = this.name;
        if (this.isModeComposition()==true) {
            out += "o";
        }
        return out;
    }
//    public static void main(String[] args) {
//        ;
//    }
}
