package jasper.element.axioms;
import jasper.element.symbols.*;
import jasper.element.*;
import java.util.*;
import jasper.data.*;
import jasper.element.operators.*;
//Axiomatic Translation, version 1.1.0, @K.J.Smith 2007-2008, http://project.kjsmith.net/
public class AxiomD extends Axiom {

    private final String name = "AxiomD";
    //Serial   []p->¬[]¬p    Ax( ¬Q[]p(x)\/Q¬[]¬p(x) )
    public AxiomD(ModalityIndex index) {
        super(index);
        this.setModeComposition(true);
    }

    //Ax( ¬Q[]p(x)\/Q¬[]¬p(x) )
    protected final Term getResult(InstantiationSet iset) {
        And returnedTerm = null;
        SymbolCache sc = this.getTM().getSymbolCache();
        FOLVariable X = sc.getVariable(VarX);
        Iterator it = iset.getBoxISet().iterator();

        while (it.hasNext()) {
            Term out = null;
            Box def = ((Box) it.next()).copy();
            Predicate symbolBoxP = def.copy().getP(); //Q[]p

            ModalityIndex r = def.getIndex().copy();
            Predicate symbolNotBoxNotP = def.copy().getTerm().negate().box(r).negate().getP(); //Q¬[]¬p

            symbolBoxP.setArg(X);
            Term symbolBoxPTerm = symbolBoxP.negate(); //¬Q[]p(x)

            symbolNotBoxNotP.setArg(X); //Q¬[]¬p(x)

            out = new Forall(new Or(symbolBoxPTerm, symbolNotBoxNotP), new QuantifierList(X));
            if (returnedTerm == null) {
                returnedTerm = new And(out);
            } else {
                returnedTerm.setTerm(out);
            }
        }
        return returnedTerm;
    }

    public Def[] getDef() { // []¬p
        ArrayList defs = new ArrayList();
        if ((this.getISet()==null) 
                || (this.getISet().getBoxISet()==null)) {
            return null;
        }
        Iterator it = this.getISet().getBoxISet().iterator();
        while (it.hasNext()) {
            defs.add(new Def(((Term) it.next()).copy()));
        }
        return (Def[]) defs.toArray(new Def[defs.size()]);
    }
    
    protected InstantiationSet getISet() {  // []¬p
        InstantiationSet is = new InstantiationSet();
        if ((this.getInstantiationSet()==null)
                ||(this.getInstantiationSet().getBoxISet()==null)) {
                return null;
            }
        Iterator it = this.getInstantiationSet().getBoxISet().iterator();
        while (it.hasNext()) {
            Term in = ((Term) it.next()).copy();
            ModalityIndex r = ((Box) in).getIndex();
            is.buildISet(((Box) in).getTerm().negate().box(r.copy()), this.getModalityIndex());
        }
        return is;
    }
    
    public final String getName() {
        String out = this.name;
        if (this.isModeComposition()==true) {
            out += "o";
        }
        return out;
    }
//    public static void main(String[] args) {
//        ;
//    }
}
