package jasper.element.axioms;
import jasper.element.symbols.*;
import jasper.element.*;
import java.util.*;
import jasper.data.*;
import jasper.element.operators.*;
//Axiomatic Translation, version 1.1.0, @K.J.Smith 2007-2008, http://project.kjsmith.net/
public class AxiomDBBB extends Axiom {
    
    private final String name = "AxiomDBBB";

    public AxiomDBBB(ModalityIndex index) {
        super(index);
        this.setModeComposition(true);
        this.setModeCf5(true);
    }

    //Ax(Ay(¬R(x,y) v ¬Q[]p(y)) v Ay(¬R(x,y) v Q[]p(y)))
    protected final Term getResult(InstantiationSet iset) {
        And returnedTerm = null;
        SymbolCache sc = this.getTM().getSymbolCache();
        FOLVariable X = sc.getVariable(VarX);
        FOLVariable Y = sc.getVariable(VarY);
        Iterator it = iset.getBoxISet().iterator();
        
        while (it.hasNext()) {
            Term out = null;
            Box def = ((Box) it.next()).copy();
            Predicate symbolBoxP = def.copy().getP(); //Q[]p
            
            AccessibilityRelation Rxy1 = sc.getAccessibilityRelation(def.getIndex().copy());
            ModalityIndex r = def.getIndex().copy();
            Rxy1.setVariables(X, Y, r); 
            Term Rxy1Term = Rxy1.negate();

            AccessibilityRelation Rxy2 = sc.getAccessibilityRelation(def.getIndex().copy());
            r = def.getIndex().copy();
            Rxy2.setVariables(X, Y, r); 
            Term Rxy2Term = Rxy2.negate();
            
            Predicate first = symbolBoxP.copy();
            first.setArg(Y); 
            
            Predicate second = symbolBoxP.copy();
            second.setArg(Y);
            Term secondTerm = second.negate(); 
            
            Term out1 = new Forall(new Or(secondTerm, Rxy1Term), new QuantifierList(Y));
            Term out2 = new Forall(new Or(first, Rxy2Term), new QuantifierList(Y));
            out = new Forall(new Or(out1, out2), new QuantifierList(X));
            if (returnedTerm==null) {
                returnedTerm = new And(out);
            } else {
                returnedTerm.setTerm(out);
            }
        }
        return returnedTerm;
    }

    public Def[] getDef() { 
        if (this.isModeCf5()==true) {
            ArrayList defs = new ArrayList();
            if ((this.getISet()==null) 
                || (this.getISet().getBoxISet()==null)) {
                return null;
            }
            Iterator it = this.getISet().getBoxISet().iterator();
            while (it.hasNext()) {
                defs.add(new Def(((Term) it.next()).copy()));
            }
            return (Def[]) defs.toArray(new Def[defs.size()]);
        } else { //no new def terms
            return null;
        }
    }

    protected InstantiationSet getISet() {  //[]¬[]p, [][]p
        InstantiationSet is = new InstantiationSet();
        if ((this.getInstantiationSet() == null) 
                || (this.getInstantiationSet().getBoxISet() == null)) {
            return null;
        }
        Iterator it = this.getInstantiationSet().getBoxISet().iterator();
        while (it.hasNext()) {
            Box in = ((Box) it.next()).copy();
            ModalityIndex r = in.getIndex();
            is.buildISet(in.negate().box(r.copy()), this.getModalityIndex());
        }
        it = this.getInstantiationSet().getBoxISet().iterator();
        while (it.hasNext()) {
            Box in = ((Box) it.next()).copy();
            ModalityIndex r = in.getIndex();
            is.buildISet(in.box(r.copy()), this.getModalityIndex());
        }
        return is;
    }
    
    public final String getName() {
        String out = this.name;
        if (this.isModeComposition()==true) {
            out += "o";
        }
        return out;
    }
//    public static void main(String[] args) {
//        ;
//    }
}
