package jasper.element.axioms;
import jasper.element.symbols.*;
import jasper.element.*;
import java.util.*;
import jasper.data.*;
import jasper.element.operators.*;
//Axiomatic Translation, version 1.1.0, @K.J.Smith 2007-2008, http://project.kjsmith.net/
public class AxiomG extends Axiom {
    
    private final String name = "AxiomG";

    public AxiomG(ModalityIndex index) {
        super(index);
        this.setModeComposition(true);
        this.setModeCf5(true);
    }

    //Ax(Ay(¬Rxy\/¬Q[]p(y))\/Ay(¬Rxy\/¬Q[]¬p(y))
    protected final Term getResult(InstantiationSet iset) {
        And returnedTerm = null;
        SymbolCache sc = this.getTM().getSymbolCache();
        FOLVariable X = sc.getVariable(VarX);
        FOLVariable Y = sc.getVariable(VarY);
        Iterator it = iset.getBoxISet().iterator();
        
        while (it.hasNext()) {
            Term out = null;
            Box def = ((Box) it.next()).copy();
            ModalityIndex r = def.getIndex().copy();
            Predicate symbolBoxP = def.copy().getP(); //Q[]p
            Predicate symbolBoxNotP = def.copy().getTerm().negate().box(r.copy()).getP(); //Q[]¬p
            
            AccessibilityRelation Rxy1 = sc.getAccessibilityRelation(def.getIndex().copy());
            r = def.getIndex().copy();
            Rxy1.setVariables(X, Y, r); 
            Term Rxy1Term = Rxy1.negate();

            AccessibilityRelation Rxy2 = sc.getAccessibilityRelation(def.getIndex().copy());
            r = def.getIndex().copy();
            Rxy2.setVariables(X, Y, r); 
            Term Rxy2Term = Rxy2.negate();
            
            Predicate first = symbolBoxP.copy();
            first.setArg(Y);  //Q[]p(y)
            Term firstTerm = first.negate();
            
            Predicate second = symbolBoxNotP.copy();
            second.setArg(Y);
            Term secondTerm = second.negate(); //¬Q[]¬p(y)
            
            Term out1 = new Forall(new Or(firstTerm, Rxy1Term), new QuantifierList(Y));
            Term out2 = new Forall(new Or(secondTerm, Rxy2Term), new QuantifierList(Y));
            out = new Forall(new Or(out1, out2), new QuantifierList(X));
            if (returnedTerm==null) {
                returnedTerm = new And(out);
            } else {
                returnedTerm.setTerm(out);
            }
        }
        return returnedTerm;
    }

    public Def[] getDef() { //[]¬p                  
        ArrayList defs = new ArrayList();
        if ((this.getBoxNotSet() == null) || (this.getBoxNotSet().getBoxISet() == null)) {
            return null;
        }
        Iterator it = this.getBoxNotSet().getBoxISet().iterator();
        while (it.hasNext()) {
            defs.add(new Def(((Term) it.next()).copy()));
        }
        if (this.isModeCf5()==true) {
            if ((this.getISet()==null) 
                || (this.getISet().getBoxISet()==null)) {
                return (Def[]) defs.toArray(new Def[defs.size()]);
            }
            it = this.getISet().getBoxISet().iterator();
            while (it.hasNext()) {
                Box next = (Box) it.next();
                defs.add(new Def(next.copy())); //original
                ModalityIndex r = next.getIndex();
                defs.add(new Def(next.copy().getTerm().negate().box(r))); //[]¬p
            }
        }
        return (Def[]) defs.toArray(new Def[defs.size()]);
    }

    private InstantiationSet getBoxNotSet() {  // []¬p
        InstantiationSet is = new InstantiationSet();
        if ((this.getInstantiationSet()==null)
                ||(this.getInstantiationSet().getBoxISet()==null)) {
                return null;
            }
        Iterator it = this.getInstantiationSet().getBoxISet().iterator();
        while (it.hasNext()) {
            Term in = ((Term) it.next()).copy();
            ModalityIndex r = ((Box) in).getIndex();
            is.buildISet(((Box) in).getTerm().negate().box(r.copy()), this.getModalityIndex());
        }
        return is;
    }
    
    protected InstantiationSet getISet() {  //[]¬[]p, []¬[]¬p
        InstantiationSet is = new InstantiationSet();
        if ((this.getInstantiationSet() == null) 
                || (this.getInstantiationSet().getBoxISet() == null)) {
            return null;
        }
        Iterator it = this.getInstantiationSet().getBoxISet().iterator();
        while (it.hasNext()) {
            Box in = ((Box) it.next()).copy();
            ModalityIndex r = in.getIndex();
            is.buildISet(in.copy().negate().box(r.copy()), this.getModalityIndex());
        }
        it = this.getInstantiationSet().getBoxISet().iterator();
        while (it.hasNext()) {
            Box in = ((Box) it.next()).copy();
            ModalityIndex r = in.getIndex();
            is.buildISet(in.copy().getTerm().negate().box(r.copy()).negate().box(r.copy()), this.getModalityIndex());
        }
        return is;
    }
    
    public final String getName() {
        String out = this.name;
        if (this.isModeComposition()==true) {
            out += "o";
        }
        return out;
    }
//    public static void main(String[] args) {
//        ;
//    }
}
