package jasper.element.axioms;
import jasper.element.*;
import jasper.element.operators.*;
import jasper.exceptions.*;
import jasper.element.symbols.*;
import jasper.main.*;
import java.util.*;
//      Ax(Qp(x) -> PI(p, x))	
//  /\	Ax(Qp(x) -> ¬Q¬p(x))	
//  /\	Ax(Q¬p(x)-> PI(¬p, x))
//Axiomatic Translation, version 1.1.0, @K.J.Smith 2007-2008, http://project.kjsmith.net/
public class Def implements Constants {

    private Term firstTerm = null;
    private Term secondTerm = null;
    private Term secondTermRev = null;
    private Term thirdTerm = null;
    private final Term formingTerm;
    private And outTerm = null;
    private boolean withShortcut = true;
    private Predicate predSymbol = null;
    private Predicate negPredSymbol = null;
            
    public Def (Term term) {
        this.formingTerm = term;
        this.predSymbol = term.copy().getP();
        this.negPredSymbol = term.copy().negate().getP();
        this.buildTerms();
    }
    
    public Def (Term term, boolean withShortcut) {
        this(term);
        this.withShortcut=withShortcut;
    }
    
    public And getTerm () {
        if (this.secondTerm==null) {
            throw new JasperRuntimeException ();
        }
        if (outTerm == null) {
            this.outTerm = new And(this.secondTerm);
            if (firstTerm != null) {
                this.outTerm.setTerm(this.firstTerm);
            }
            if (thirdTerm != null) {
                this.outTerm.setTerm(this.thirdTerm);
            }
            if (secondTermRev != null) {
                this.outTerm.setTerm(this.secondTermRev);
            }
        }
        return this.outTerm;
    }
    
    public Term getFormingTerm () {
        return this.formingTerm.copy();
    }
   
    private void buildTerms () {
        TranslationManager tm = TranslationManager.getTransationManager();
        Predicate one;
        Term two;
        FOLVariable X = tm.getSymbolCache().getVariable(VarX);
        FOLVariable Y = tm.getSymbolCache().getVariable(VarY);
        { //   	Ax(Qp(x) -> ¬Q¬p(x))
            one = this.predSymbol.copy(); //Qp(x)
            one.setArg(X);
            two = (Predicate) this.negPredSymbol.copy(); //¬Q¬p(x)
            ((Predicate)two).setArg(X);
            two = two.negate();
            this.secondTerm = new Forall(new Implies(one, two), new QuantifierList(X));
        } //   	Ax(Qp(x) <- ¬Q¬p(x))
        if (this.withShortcut==true) {
            one = this.predSymbol.copy();
            one.setArg(X);
            two = (Predicate)this.negPredSymbol.copy();
            ((Predicate)two).setArg(X);
            two = two.negate();
            this.secondTermRev = new Forall(new Implied(one, two), new QuantifierList(X));    
        }
        if (!(this.formingTerm instanceof ModalPredicate)) {
            {//      Ax(Qp(x) -> PI(p, x))
                one = this.predSymbol.copy();
                one.setArg(X);
                this.firstTerm = new Implies(one, 
                        this.pi(this.formingTerm.copy(),X,Y));
                this.firstTerm = new Forall(this.firstTerm, new QuantifierList(X));
            }
            { //    	Ax(Q¬p(x)-> PI(¬p, x))
                two = this.negPredSymbol.copy();
                ((Predicate)two).setArg(X);
                this.thirdTerm = new Implies(two, 
                        this.pi(this.formingTerm.copy().negate(),X,Y));
                this.thirdTerm = new Forall(this.thirdTerm, new QuantifierList(X));
            }
        }
        return;
    } 
    
    private Term pi(Term term, FOLVariable X, FOLVariable Y) {
        // a. PI(false, x)  = false 
        if (term instanceof False) {
            return new False();
        } // b. PI(p, x) = T  
        else if (term instanceof ModalPredicate) {
            return new True();
        } // c. PI(¬p, x) = ¬Qp(x) 
        else if ( (term instanceof Not) 
                && (((Not)term).getTerm() instanceof ModalPredicate)) {
            Predicate Qp = term.copy().getP();
            Qp.setArg(X); // ¬Qp(x)
            return Qp.negate();
	} // d. PI(p/\q, x) = Qp(x) /\ Qq(x) 
	else if (term instanceof And) {
            Iterator it = ((And)term.copy()).getTerm().iterator();
            ArrayList out = new ArrayList();
            while (it.hasNext()) {
                Predicate Qvar = ((Term)it.next()).getP();
                Qvar.setArg(X); // Qp(x), Qq(x), etc 
		out.add(Qvar);
            }
            return new And(out);
        } // e. PI(¬(p/\q), x) = Q¬p(x) \/ Q¬q(x) 
	else if ((term instanceof Not) 
                && (((Not)term).getTerm() instanceof And)) {
            Iterator it = ((And)((Not)term.copy()).getTerm()).getTerm().iterator();
            ArrayList out = new ArrayList();
            while (it.hasNext()) {
                Term tmp = (Term)it.next();
                Predicate Qvar = tmp.negate().getP();
                Qvar.setArg(X); // Q¬p(x), Q¬q(x), etc 
		out.add(Qvar);
            }
            return new Or(out);
	} // f. PI([](r,p), x) = Ay( R(x, y) -> Qp(y) ) 
	else if (term instanceof Box) {
            Predicate Qp = ((Box)term.copy()).getTerm().getP();
            Qp.setArg(Y);
            ModalityIndex r = ((Box)term).getIndex().copy();
            AccessibilityRelation Rxy = new AccessibilityRelation("R",X,Y,r);
            return new Forall(new Implies(Rxy,Qp), new QuantifierList(Y));
	} // g. PI(¬[](r,p), x) = Ey( R(x, y) /\ Q¬p(y) ) */
	else if ( (term instanceof Not) 
                && (((Not)term).getTerm() instanceof Box) ) {
            Predicate Qnp = ((Box)((Not)term.copy()).getTerm()).getTerm().negate().getP();
            Qnp.setArg(Y); // Q¬p(y) 
            ModalityIndex r = ((Box)((Not)term).getTerm()).getIndex();
            AccessibilityRelation Rxy = new AccessibilityRelation("R",X,Y,r);
            return new Exists(new And(Rxy, Qnp), new QuantifierList(Y));
        } else {
		throw new JasperRuntimeException(term.toString());
        }
    }

    public boolean equals (Def def) {
        if ((this.formingTerm.equals(def.getFormingTerm()))==true) {
            return true;
        } else {
            return false;
        }
    }
    
//    public static void main(String[] args) {
//        ;
//    }
}
