package jasper.element.symbols;
import jasper.element.interfaceAbstract.*;
import jasper.element.*;
import java.util.*;
import jasper.exceptions.*;
//Axiomatic Translation, version 1.1.0, @K.J.Smith 2007-2008, http://project.kjsmith.net/
//Rr(X,Y)
public class AccessibilityRelation extends BinaryPredicate implements Binary {

    private FOLVariable variable1 = null; 
    private FOLVariable variable2 = null;
    private ModalityIndex modalityIndex = null;
    
    public AccessibilityRelation(String name, FOLVariable X, FOLVariable Y, ModalityIndex r) {
        super(name);
        this.setVariables (X, Y, r);
    }
    
    public void setVariables (FOLVariable X, FOLVariable Y, ModalityIndex r) {
        if ((X==null)||(Y==null)||(r==null)) { 
            throw new JasperRuntimeException();
        }
        this.variable1 = X;
        this.variable2 = Y;
        this.modalityIndex = r;
    }
    
    public ArrayList descendList() {
        ArrayList out = new ArrayList();
        out.add(this);
        return out;
    }
    
    public boolean equals(Term term) {
        if (term==null) {
            throw new JasperRuntimeException();
        }
        if ( (term instanceof AccessibilityRelation)
            && (this.getName().equals(term.getName())==true)
            && (this.variable1.equals(((AccessibilityRelation)term).getVariable1())==true) 
            && (this.variable2.equals(((AccessibilityRelation)term).getVariable2())==true)
            && (this.modalityIndex.equals(((AccessibilityRelation)term).getModalityIndex())==true) ) {
            return true;
        } else {
            return false;
        }
    }

    public String toString() {
        String str = this.getName();
        str += this.getModalityIndex().getName()+"(";
        str += this.variable1.toString()+",";
        str += this.variable2.toString()+")";
        return str;
    }

    public String inString() {
        String str = ""+this.getNemonic();
        str += this.getModalityIndex().getNemonic()+"(";
        str += this.variable1.inString()+",";
        str += this.variable2.inString()+")";
        return str;
    }
    
    public String asString() {
        String str = this.getName();
        str += this.getModalityIndex().getName()+"_";
        str += this.variable1.toString()+",";
        str += this.variable2.toString()+"_";
        return str;
    }
    
    public AccessibilityRelation copy() {
        return new AccessibilityRelation(this.getName(), 
                this.variable1.copy(), this.variable2.copy(), 
                this.modalityIndex.copy());
    }

    public ModalityIndex getModalityIndex() {
        return this.modalityIndex;
    }

    public FOLVariable getVariable1() {
        return this.variable1;
    }

    public FOLVariable getVariable2() {
        return this.variable2;
    }
    
//    public static void main(String[] args) {
//        ;
//    }
}
