package jasper.data;
import jasper.element.operators.*;
import jasper.element.symbols.*;
import java.util.*;
import jasper.element.*;
import jasper.exceptions.*;
//Axiomatic Translation, version 1.1.0, @K.J.Smith 2007-2008, http://project.kjsmith.net/
public class InstantiationSet {

    private final ArrayList iset = new ArrayList();

    public InstantiationSet() {
    }

    public void buildISet(final Term term, ModalityIndex r) {
        Term descendingTerm = term.copy();
        Iterator it = descendingTerm.descendList().iterator();
        while (it.hasNext()) {
            descendingTerm = (Term) it.next();
            if (r.equals(ModalityIndex.dummy)) {
                this.build(descendingTerm);
            } else if ( (descendingTerm instanceof Box) 
                    && (r.equals(((Box)descendingTerm).getIndex())) ) {
                this.build(descendingTerm);
            }
        }
    }

    private void build(final Term term) {
        if (term instanceof Box) {
            this.buildBox(term);
        } else if (term instanceof And) {
            this.buildAnd(term);
        } else if (term instanceof ModalPredicate) {
            this.buildPredicate(term);
//        } else if (term instanceof Not) {
//            this.buildNot(term);
        } else {
            //nothing
        }
    }

    public InstantiationSet plus (InstantiationSet is) {
        if (is!=null) {
            Iterator it = is.getISet().iterator();
            while (it.hasNext()) {
                Term tmp = (Term)it.next();
                this.build(tmp);
            }
        }
        return this;
    }
    
    @Override
    public String toString() {
        String str = "";
        if (this.iset.isEmpty() == true) {
            throw new JasperRuntimeException();
        }
        Iterator it = this.iset.iterator();
        while (it.hasNext()) {
            str += ((Term) it.next()).toString();
            if (it.hasNext()) {
                str += ",";
            }
        }
        return str;
    }

    private boolean contains(Term term) {
        boolean out = false;
        Iterator it = this.iset.iterator();
        while (it.hasNext()) {
            Term tmp = (Term) it.next();
            //tmp.print();
            if (term.equals(tmp)) {
                out = true;
            }
        }
        return out;
    }

    private void buildBox(Term term) {
        if (this.contains(term) == false) {
            this.iset.add(term);
        }
    }

    private void buildAnd(Term term) {
        if (this.contains(term) == false) {
            this.iset.add(term);
        }
    }

    private void buildPredicate(Term term) {
        if (this.contains(term) == false) {
            this.iset.add(term);
        }
    }

    private void buildNot(Term term) {
        if (this.contains(term) == false) {
            this.iset.add(term);
        }
    }

    private void addToISet(Term term) {
        if (this.contains(term) == false) {
            this.iset.add(term);
        }
    }

    public InstantiationSet copy() {
        InstantiationSet is = new InstantiationSet();
        Iterator it = this.iset.iterator();
        while (it.hasNext()) {
            is.addToISet(((Term) it.next()).copy());
        }
        return is;
    }

    public ArrayList getISet() {
        if (this.isInitialized() == true) {
            return new ArrayList(this.iset);
        } else {
            throw new UnsupportedOperationException();
        }
    }

    public ArrayList getBoxISet() {
        if (this.isInitialized() == true) {
            ArrayList list = new ArrayList();
            Iterator it = this.iset.iterator();
            while (it.hasNext()) {
                Term tmp = ((Term) it.next()).copy();
                if (tmp instanceof Box) {
                    list.add(tmp);
                }
            }
            return list;
        } else {
            throw new UnsupportedOperationException();
        }
    }

    public boolean isInitialized() {
        if (this.iset.isEmpty() == true) {
            return false;
        } else {
            return true;
        }
    }

//    public static void main(String[] args) {
//        ;
//    }
}
