package jasper.data;
import jasper.main.*;
import java.util.*;
import jasper.element.symbols.*;
import jasper.exceptions.*;
//Axiomatic Translation, version 1.1.0, @K.J.Smith 2007-2008, http://project.kjsmith.net/
public class SymbolCache implements Constants {

    private final ArrayList predicates = new ArrayList();
    private final ArrayList variables = new ArrayList();
    private final ArrayList translpairs = new ArrayList();
    private final ArrayList nameRecordedList = new ArrayList();
    private final ArrayList nameSearchList = new ArrayList();
    //private String globalR = "R";
    
    public SymbolCache() {
        this.variables.add(new FOLVariable("U")); //0 X ... U
        this.variables.add(new FOLVariable("V")); //1 Y ... V
        this.variables.add(new FOLVariable("W")); //2 Z ... W
        this.variables.add(new FOLVariable("X")); //3 U ... X
        this.variables.add(new FOLVariable("Y")); //4 V ... Y
        this.variables.add(new FOLVariable("Z")); //5
        this.variables.add(new FOLVariable("A")); //6
        this.variables.add(new FOLVariable("B")); //7
        this.variables.add(new FOLVariable("C")); //8
        this.translpairs.add(new SymbolCache.Translpair("R", "r"));
        this.translpairs.add(new SymbolCache.Translpair("R", "s"));
        this.translpairs.add(new SymbolCache.Translpair("R", "a"));
        this.addPredicate(new Predicate("p"));
        this.addPredicate(new Predicate("q"));
    }
    public String getFromNameList (String searchStr) {
        String out = null;
        
        for (int i = 0; i < this.nameSearchList.size(); i++) {
            String nxt = (String)this.nameSearchList.get(i);
            if (nxt.equals(searchStr)) {
                out = (String)this.nameRecordedList.get(i);
            }
        }
        return out;
    }
    
    public void recordName (String searchStr, String recordedStr) {
        String test = this.getFromNameList(searchStr);
        if (test!=null) {
            throw new JasperRuntimeException();
        }
        this.nameRecordedList.add(recordedStr);
        this.nameSearchList.add(searchStr);
    }
    
    public void namesReset() {
        this.nameRecordedList.clear();
        this.nameSearchList.clear();
    }
    
    public Predicate getPredicate(String str) {
        Predicate out = null;
        Iterator it = this.predicates.iterator();
        while (it.hasNext()) {
            Predicate tmp = (Predicate)it.next();
            if (tmp.getName().equals(str)) {
                out = tmp;
            }
        }
        return out.copy();
    }

    private void addPredicate(Predicate predicate) {
        Iterator it = this.predicates.iterator();
        while (it.hasNext()) {
            Predicate tmp = (Predicate)it.next();
            if (tmp.equals(predicate)) {
                return;
            }
        }
        this.predicates.add(predicate);
    }

    public FOLVariable getVariable(int code) {
        FOLVariable out;
        try {
            out = (FOLVariable) this.variables.get(code);
        } catch (IndexOutOfBoundsException e) {
            for (int i = this.variables.size(); i <= code; i++) {
                this.variables.add(new FOLVariable("V" + i));
            }
            out = (FOLVariable) this.variables.get(code);
        }
        return out;
    }

    public void addTranslpair(AccessibilityRelation accessRelation, ModalityIndex modalityIndex) {
        Iterator it = translpairs.iterator();
        while (it.hasNext()) {
            Translpair tp = (Translpair)it.next();
            if (tp.getModalityIndex().equals(modalityIndex)) {
                throw new JasperRuntimeException();
            }
            if (tp.getAccessibilityRelation().equals(accessRelation)) {
                throw new JasperRuntimeException();
            }
        }
        translpairs.add(new Translpair(accessRelation, modalityIndex));
    }
    
    public AccessibilityRelation getAccessibilityRelation(ModalityIndex modalityIndex) {
        AccessibilityRelation ar = null;
        Iterator it = translpairs.iterator();
        while (it.hasNext()) {
            Translpair tp = (Translpair)it.next();
            if (tp.getModalityIndex().equals(modalityIndex)) {
                ar = tp.getAccessibilityRelation().copy();
            }
        }
        if (ar==null) {
            String strAR = modalityIndex.getName().toUpperCase();
            String strMI = modalityIndex.getName().toLowerCase();
            it = translpairs.iterator();
            while (it.hasNext()) {
                Translpair tp = (Translpair)it.next();
                if (tp.getModalityIndex().getName().equals(strMI)) {
                    throw new JasperRuntimeException();
                }
                if (tp.getAccessibilityRelation().getName().equals(strAR)) {
                    throw new JasperRuntimeException();
                }
            }
            translpairs.add(new Translpair(
                    new AccessibilityRelation(strAR, FOLVariable.dummy, FOLVariable.dummy, ModalityIndex.dummy), 
                    new ModalityIndex(strMI)));
            ar = new AccessibilityRelation(strAR, FOLVariable.dummy, FOLVariable.dummy, ModalityIndex.dummy);
        }
        return ar;
    }

    public ModalityIndex getModalityIndex(AccessibilityRelation accessRelation) {
        ModalityIndex mi = null;
        Iterator it = translpairs.iterator();
        while (it.hasNext()) {
            Translpair tp = (Translpair)it.next();
            if (tp.getAccessibilityRelation().equals(accessRelation)) {
                mi = tp.getModalityIndex().copy();
            }
        }
        if (mi==null) {
            String strAR = accessRelation.getName().toUpperCase();
            String strMI = accessRelation.getName().toLowerCase();
            it = translpairs.iterator();
            while (it.hasNext()) {
                Translpair tp = (Translpair)it.next();
                if (tp.getModalityIndex().getName().equals(strMI)) {
                    throw new JasperRuntimeException();
                }
                if (tp.getAccessibilityRelation().getName().equals(strAR)) {
                    throw new JasperRuntimeException();
                }
            }
            translpairs.add(new Translpair(
                    new AccessibilityRelation(strAR, FOLVariable.dummy, FOLVariable.dummy, ModalityIndex.dummy), 
                    new ModalityIndex(strMI)));
            mi = new ModalityIndex(strMI);
        }
        return mi;
    }
    
    //-------------------------------------------
    private class Translpair {

        private final AccessibilityRelation accessRelation;
        private final ModalityIndex index;

        private Translpair(AccessibilityRelation accessRelation, ModalityIndex index) {
            this.accessRelation = accessRelation;
            this.index = index;
        }

        private Translpair(String arelation, String index) {
            this.accessRelation = new AccessibilityRelation(
                    arelation, FOLVariable.dummy, FOLVariable.dummy, ModalityIndex.dummy);
            this.index = new ModalityIndex(index);
        }

        private AccessibilityRelation getAccessibilityRelation() {
            return this.accessRelation;
        }

        private ModalityIndex getModalityIndex() {
            return this.index;
        }
    }
//    public static void main(String[] args) {
//        ;
//    }
}
