package jasper.element;
import jasper.element.operators.*;
import jasper.exceptions.*;
import java.util.*;
import jasper.element.interfaceAbstract.*;
//Axiomatic Translation, version 1.1.0, @K.J.Smith 2007-2008, http://project.kjsmith.net/
public abstract class BinaryTerm extends Term implements Binary {

    private final Term term1;
    private final Term term2;

    public BinaryTerm(Term term1, Term term2, String name)
            throws TermCompositionException {
        super(name);
        if ((term1==null)||(term2==null)) { 
            throw new JasperRuntimeException();
        }
        this.term1 = term1;
        this.term2 = term2;
        if (this.checkTypeOK() == false) {
            throw new TermCompositionException();
        }
        this.setNemonic(this.nemonic);
    }

    public ArrayList descendList() {
        ArrayList out = new ArrayList();
        out.add(this);
        out.addAll(this.term1.descendList());
        out.addAll(this.term2.descendList());
        return out;
    }

    public boolean equals(Term term) {
        if (term==null) {
            throw new JasperRuntimeException();
        }
        if ((term instanceof BinaryTerm) 
                && (this.term1.equals(((BinaryTerm)term).getFirstTerm())==true)
                && (this.term2.equals(((BinaryTerm)term).getSecondTerm())==true) ) { 
            return true;
        } else {
            return false;
        }
    }
    
    public Term[] getTerm() {
        return new Term[]{this.getFirstTerm(), this.getSecondTerm()};
    }

    public Term getFirstTerm() {
        return this.term1.copy();
    }

    public Term getSecondTerm() {
        return this.term2.copy();
    }

    protected boolean checkTypeOK() {
        if (this.getFirstTerm() instanceof QuantifierList) {
            return false;
        } else if (this.getSecondTerm() instanceof QuantifierList) {
            return false;
        } else if (this.getFirstTerm() instanceof TermElement) {
            return false;
        } else if (this.getSecondTerm() instanceof TermElement) {
            return false;
        } else {
            return true;
        }
    }
    
    public String toString() {
        return this.getName() + "(" 
                + this.term1.toString() + "," 
                + this.term2.toString() + ")";
    }
    
    public String inString() {
        return this.getNemonic() + "(" 
                + this.term1.inString() + "," 
                + this.term2.inString() + ")";
    }
    
    public String asString() {
        return this.getName() + "_" 
                + this.term1.asString() + "_" 
                + this.term2.asString();
    }
    
//    public static void main(String[] args) {
//        ;
//    }
}
