package jasper.element;
import jasper.element.interfaceAbstract.BoolTerm;
import java.util.*;
import jasper.element.symbols.*;
import jasper.element.operators.*;
//Axiomatic Translation, version 1.1.0, @K.J.Smith 2007-2008, http://project.kjsmith.net/
public abstract class Term {

    private final String name;
    protected char nemonic = '\u0000';
    private String description = "";

    protected Term (String name) {
        if (name==null) {
            name = "null";
        }
        this.name = name;
    } 
    
    protected boolean checkTypeOK() {
       return true;
    }
    
    public String getName () {
        return this.name;
    }
    
    public char getNemonic () {
        return this.nemonic;
    }
    public Term setNemonic (char nemonic) {
        this.nemonic = nemonic;
        return this;
    }
    
    public String getDescription() {
        return this.description;
    }
    public Term setDescription(String description) {
        this.description = description;
        return this;
    }
    
    public abstract ArrayList descendList();
    
    public abstract boolean equals (Term term);
        
    public void print () {
        System.out.println(toString());
    }
    
    @Override
    public abstract String toString ();
    
    public abstract String asString ();
    
    public abstract String inString ();
    
    public Term negate() {
        return this.negateEliminateNotNot(this);
    }
    
    public Term forceNegate() {
        return this.negateOnly(this);
    }
    
    public Term complement () {
        throw new UnsupportedOperationException("Not supported yet.");
    }
    
    public Term box (ModalityIndex index) {
        return new Box(index, this);
    }
    
    public Term diamond (ModalityIndex index) {
        return this.copy().box(index).negate().box(index);
    }
    
    public Term forceDiamond (ModalityIndex index) {
        return new Diamond(index, this);
    }
    
    public Term forall (QuantifierList qlist) {
        return new Forall(this, qlist);
    }
    
    public Term exists (QuantifierList qlist) {
        return new Forall(this, qlist);
    }
    
    public Term simplify () {
        throw new UnsupportedOperationException("Not supported yet.");
    }
    
    public abstract Term copy ();
    
    //-------------------------------------------
        
    public Term simplfyAnds () {
        boolean changed = true;
        Term outTerm = this.copy();
        while (changed==true) {
            Object[] result = Ands(outTerm); 
            changed = (Boolean)result[0];
            outTerm = (Term)result[1];
        }
        return outTerm;
    }
    
    private Object[] Ands (Term term) {
        Object[] out = new Object[2];
        out[0] = false;
        
        if (term instanceof And) {
            ArrayList list = new ArrayList();
            Iterator it = ((And)term).getTerm().iterator();
            while (it.hasNext()) {
                Term termTmp = (Term)it.next();
                if (termTmp instanceof And) {
                    list.addAll(((And)termTmp).getTerm());
                    out[0] = true;
                } else {
                    Object[] tmp = Ands(termTmp);
                    if ((Boolean)tmp[0]==true) {
                        out[0] = true;
                    }
                    list.add(tmp[1]);
                }
            }
            out[1] = new And(list);
        } else if (term instanceof Or) {
            ArrayList list = new ArrayList();
            Iterator it = ((Or)term).getTerm().iterator();
            while (it.hasNext()) {
                Term termTmp = (Term)it.next();
                if (termTmp instanceof Or) {
                    list.addAll(((Or)termTmp).getTerm());
                    out[0] = true;
                } else {
                    Object[] tmp = Ands(termTmp);
                    if ((Boolean)tmp[0]==true) {
                        out[0] = true;
                    }
                    list.add(tmp[1]);
                }
            }
            out[1] = new Or(list);
//            ArrayList list = new ArrayList();
//            Iterator it = ((Or)term).getTerm().iterator();
//            Object[] tmp;
//            while (it.hasNext()) {
//                tmp = Ands((Term)it.next());
//                if ((Boolean)tmp[0]==true) {
//                    out[0] = true;
//                }
//                list.add(tmp[1]);
//            }
//            out[1] = new Or(list);
        } else if (term instanceof BinaryTerm) {
            Term[] pair = new Term[2];
            Object[] tmp;
            tmp = Ands(((BinaryTerm)term).getFirstTerm());
            pair[0] = (Term)tmp[1];
            if ((Boolean)tmp[0]==true) {
                out[0] = true;
            }
            tmp = Ands(((BinaryTerm)term).getSecondTerm());
            pair[1] = (Term)tmp[1];
            if ((Boolean)tmp[0]==true) {
                out[0] = true;
            }
            if (term instanceof Implies) {
                out[1] = new Implies(pair[0], pair[1]);
            } else if (term instanceof Implied) {
                out[1] = new Implied(pair[0], pair[1]);
            } else if (term instanceof Equiv) {
                out[1] = new Equiv(pair[0], pair[1]);
            }
        } else if (term instanceof QuantifierList) {
            out[1] = term;
        } else if (term instanceof Not) {
            out = Ands(((Not)term).getTerm());
            out[1] = new Not((Term)out[1]);
        } else if (term instanceof QuantifierTerm) {
            Object[] tmp;
            QuantifierList ql = ((QuantifierTerm)term).getQuantifierList();
            tmp = Ands(((QuantifierTerm)term).getTerm());
            
            if (term instanceof Forall) {
                out[1] = new Forall((Term)tmp[1], ql);
            } else if (term instanceof Exists) {
                out[1] = new Exists((Term)tmp[1] ,ql);
            }
        } else if (term instanceof ModalTerm) {
            Object[] tmp;
            ModalityIndex index = ((ModalTerm)term).getIndex();
            tmp = Ands(((ModalTerm)term).getTerm());
            out[0] = tmp[0];
            if (term instanceof Box) {
                out[1] = new Box(index, (Term)tmp[1]);
            } else if (term instanceof Diamond) {
                out[1] = new Diamond(index, (Term)tmp[1]);
            }
        } else if (term instanceof FOLVariable) {
            out[1] = term;
        } else if (term instanceof ModalPredicate) {
            out[1] = term;
        } else if (term instanceof ModalityIndex) {
            out[1] = term;
        } else if (term instanceof Predicate) {
            out[1] = term;
        } else if (term instanceof BoolTerm) {
            out[1] = term;
        } else if (term instanceof AccessibilityRelation) {
            out[1] = term;
        } else {
            throw new RuntimeException(term.toString());
        }
        return out;
    }
    
    private Term negateOnly (Term term) {
        return new Not(term);
    }
   
    public Predicate getP(){
        return this.getP("Q");
    }
    public Predicate getP(String q){
        return new Predicate(q+this.asString());
    }
    
    private Term negateEliminateNotNot (Term term) {
        if (!(term instanceof Not)) {
            return this.negateOnly(term);
        } else {
            return ((UnaryTerm)term).getTerm();
        }
    }
    
    private Term removeTrivialAtomsAndOps() {
        throw new UnsupportedOperationException("Not supported yet.");
    }
    
    private Term simplifyQuantors() {
        throw new UnsupportedOperationException("Not supported yet.");
    }	
    
    private Term eliminateDiamond() {
        throw new UnsupportedOperationException("Not supported yet.");
    }
    
    private Term eliminateTrue() {
        throw new UnsupportedOperationException("Not supported yet.");
    }
    
    private Term eliminateModals() {
        throw new UnsupportedOperationException("Not supported yet.");
    }
    
    private Term eliminateNotNot() {
        throw new UnsupportedOperationException("Not supported yet.");
    }
    
    private Term eliminateBiImplication() {
        throw new UnsupportedOperationException("Not supported yet.");
    }
    
    private Term eliminateImplied() {
        throw new UnsupportedOperationException("Not supported yet.");
    }
    
    private Term eliminateImplies() {
        throw new UnsupportedOperationException("Not supported yet.");
    }
    
    private Term eliminateOr() {
        throw new UnsupportedOperationException("Not supported yet.");
    }
    
    private Term eliminateBoxAnd() {
        throw new UnsupportedOperationException("Not supported yet.");
    }
    
    private Term eliminateNestedAnd() {
        throw new UnsupportedOperationException("Not supported yet.");
    }
    
//    public static void main(String[] args) {
//        ;
//    }
}
