package jasper.element;
import jasper.element.interfaceAbstract.*;
import jasper.element.operators.*;
import jasper.exceptions.*;
import java.util.*;
//Axiomatic Translation, version 1.1.0, @K.J.Smith 2007-2008, http://project.kjsmith.net/
public abstract class UnaryTerm extends Term implements Unary {

    private final Term term;

    public UnaryTerm(Term term, String name)
            throws TermCompositionException {
        super(name);
        this.term = term;
        if (term==null) { 
            throw new JasperRuntimeException();
        }
        if (this.checkTypeOK() == false) {
            throw new TermCompositionException();
        }
        this.setNemonic(this.nemonic);
    }

    protected boolean checkTypeOK() {
        if (this.term instanceof TermElement) {
            return false;
        } else if (this.term instanceof QuantifierList) {
            return false;
        } else if (this.term instanceof Quantifier) {
            return false;
        } else {
            return true;
        }
    }

    public boolean equals(Term term) {
        if (term==null) {
            throw new JasperRuntimeException();
        }
        if ((term instanceof UnaryTerm) 
                && (this.getTerm().equals(((UnaryTerm)term).getTerm())==true) ) {
            return true;
        } else {
            return false;
        }
    }
    
    public ArrayList descendList() {
        ArrayList out = new ArrayList();
        out.add(this);
        out.addAll(this.term.descendList());
        return out;
    }

    public final Term getTerm() {
        return this.term.copy();
    }

    public String toString() {
        return this.getName() + "(" + this.getTerm().toString() + ")";
    }
    
    public String inString() {
        return this.getNemonic() + "(" + this.getTerm().inString() + ")";
    }
    
    public String asString() {
        return this.getName() + "_" + this.getTerm().asString();
    }
    
//    public static void main(String[] args) {
//        ;
//    }
}
