package jasper.element.axioms;
import jasper.element.*;
import jasper.data.*;
import java.util.*;
import jasper.element.symbols.*;
import jasper.main.*;
import jasper.exceptions.*;
import jasper.element.operators.*;
//Axiomatic Translation, version 1.1.0, @K.J.Smith 2007-2008, http://project.kjsmith.net/
public abstract class Axiom implements Constants {

    private static final TranslationManager tm = TranslationManager.getTransationManager();
    private String name = null;
    private ModalityIndex index = null;
    private boolean modeComposition = false; //default
    private boolean modeCf5 = false; //default
//    private boolean modeClassical = false; //default
    private InstantiationSet instset = null;
    
    public Axiom(ModalityIndex index) {
        if (index==null) {
            throw new JasperRuntimeException();
        }
        this.index = index;
    }

    /*
     * Provide Instantiation set as the basis of translation
     */
    public void setInstantiationSet (InstantiationSet is) {
        if (this.instset==null) {
            if (this.index.equals(ModalityIndex.dummy) ) { //taking all
                this.instset = is.copy();
            } else { //use an equals test
                this.instset = new InstantiationSet();
                Iterator it = is.getBoxISet().iterator();
                while (it.hasNext()) {
                    Box content = (Box)it.next();
                    ModalityIndex r = content.getIndex();
                    if (r.equals(this.index)) {
                        this.instset.buildISet(content, this.getModalityIndex());
                    }
                }
            }
        } else {
            throw new JasperRuntimeException();
        }
    }
    
    /*
     * Instantiation set **provided** as the basis of translation
     * in setInstantiationSet()
     */
    public InstantiationSet getInstantiationSet() {
        return this.instset.copy();
    }
    
    /*
     * access and control, setting appropriate instantiation set, 
     * to get AXIOM(is)
     */
    public final Term getResult() {
        if (this.isModeCf5()==true) {
            //System.out.println("QQQQQQQQQQQQQQQQ");
            InstantiationSet isadd = this.getInstantiationSet();
            isadd.plus(this.getCompositionalISet());
            return this.getResult(isadd);
        } else {
            return this.getResult(this.getInstantiationSet());
        }
    }
    
    /*
     * Do ACTUAL translation for given instantiation set
     */
    protected abstract Term getResult(InstantiationSet iset);
    
    /*
     * return Defs from NEW terms
     */
    public abstract Def[] getDef();

    /*
     * return Defs from Compositional terms
     */
    public final Def[] getCompositionalDef() {
        if ((this.isModeComposition()==true)
          ||(this.isModeCf5()==true)){
            ArrayList defs = new ArrayList();
            if ((this.getCompositionalISet()==null)
                    || (this.getCompositionalISet().getBoxISet()==null)) {
                    return null;
            }
            Iterator it = this.getCompositionalISet().getBoxISet().iterator();
            while (it.hasNext()) {
               defs.add(new Def((Term) it.next()));
            }
            return (Def[])defs.toArray(new Def[defs.size()]);
        } else {
            return null;
        }
    }
    
    /*
     * Instantiation set calculated for passing over to **next** axiom 
     * or **this and next** axiom in cf5 mode
    */
    public final InstantiationSet getCompositionalISet() {
        if ((this.isModeComposition()==true) || (this.isModeCf5()==true)) {
            return this.getISet();
        } else {
            return null;
        }
    }
    
    protected abstract InstantiationSet getISet();
    //////////////////////////////////////////////////
    protected TranslationManager getTM() {
        return Axiom.tm;
    }
    
    public boolean isModeCf5() {
        return this.modeCf5;
    }

    public void setModeCf5(boolean modeCf5) {
        this.modeCf5 = modeCf5;
        if (this.modeCf5==true) {
            this.modeComposition = true;
        }
    }

    public boolean isModeComposition() {
        return this.modeComposition;
    }

    public void setModeComposition(boolean modeComposition) {
        this.modeComposition = modeComposition;
        if (this.modeComposition==false) {
            this.modeCf5=false;
        }
    }
    
//    public boolean isModeClassical() {
//        return this.modeClassical;
//    }

//    public void setModeClassical(boolean modeClassical) {
//        this.modeClassical = modeClassical;
//    }

    public final ModalityIndex getModalityIndex() {
        return this.index;
    }

    public abstract String getName();

    @Override
    public final String toString() {
        return this.getName();
    }
    
//    public static void main(String[] args) {
//        ;
//    }
}
