package jasper.element.axioms;
import jasper.element.symbols.*;
import jasper.element.*;
import java.util.*;
import jasper.data.*;
import jasper.element.operators.*;
//Axiomatic Translation, version 1.1.0, @K.J.Smith 2007-2008, http://project.kjsmith.net/
public class Axiom53 extends Axiom {
    
    private final String name = "Axiom53";
    
    public Axiom53(ModalityIndex index) {
        super(index);
        this.setModeComposition(true);
        this.setModeCf5(true);
    }

    //5^3   ¬[][][]¬[]p -> []p   
    // Ax( Q[]rp(x) \/ Azuy(¬Rxz \/ ¬Rzu \/ ¬Ruy \/ ¬Q[]rp(y)) )
    protected final Term getResult(InstantiationSet iset) {
        And returnedTerm = null;
        SymbolCache sc = this.getTM().getSymbolCache();
        FOLVariable X = sc.getVariable(VarX);
        FOLVariable Y = sc.getVariable(VarY);
        FOLVariable Z = sc.getVariable(VarZ);
        FOLVariable U = sc.getVariable(VarU);
        Iterator it = iset.getBoxISet().iterator();
        
        while (it.hasNext()) {
            Term out = null;
            Box def = ((Box) it.next()).copy();
            Predicate symbolBoxP = def.copy().getP(); //Q[]p
            
            AccessibilityRelation Rxz = sc.getAccessibilityRelation(def.getIndex().copy());
            ModalityIndex r = def.getIndex().copy();
            Rxz.setVariables(X, Z, r); //Rxy
            Term RxzTerm = Rxz.negate();

            AccessibilityRelation Rzu = sc.getAccessibilityRelation(def.getIndex().copy());
            r = def.getIndex().copy();
            Rzu.setVariables(Z, U, r); //Rxy
            Term RzuTerm = Rzu.negate();

            AccessibilityRelation Ruy = sc.getAccessibilityRelation(def.getIndex().copy());
            r = def.getIndex().copy();
            Ruy.setVariables(U, Y, r); //Rxy
            Term RuyTerm = Ruy.negate();
            
            Predicate first = symbolBoxP.copy();
            first.setArg(X);  //Q[]p(x)
            
            Predicate second = symbolBoxP.copy();
            second.setArg(Y);
            Term secondTerm = second.negate(); //¬Q[]p(y)
            
            //Ax( Q[]rp(x) \/ Azuy(¬Rxz \/ ¬Rzu \/ ¬Ruy \/ ¬Q[]rp(y)) )
            out = new Forall(new Or(secondTerm, RxzTerm, RzuTerm, RuyTerm), new QuantifierList(Z,U,Y));
            out = new Forall(new Or(first, out), new QuantifierList(X));
            if (returnedTerm==null) {
                returnedTerm = new And(out);
            } else {
                returnedTerm.setTerm(out);
            }
        }
        return returnedTerm;
    }

    public Def[] getDef() { 
        if (this.isModeCf5()==true) {
            ArrayList defs = new ArrayList();
            if ((this.getISet()==null) 
                || (this.getISet().getBoxISet()==null)) {
                return null;
            }
            Iterator it = this.getISet().getBoxISet().iterator();
            while (it.hasNext()) {
                defs.add(new Def(((Term) it.next()).copy()));
            }
            return (Def[]) defs.toArray(new Def[defs.size()]);
        } else { //no new def terms
            return null;
        }
    }

    protected InstantiationSet getISet() {  //   53     [][][]¬[]p
        InstantiationSet is = new InstantiationSet();
        if ((this.getInstantiationSet() == null) 
                || (this.getInstantiationSet().getBoxISet() == null)) {
            return null;
        }
        Iterator it = this.getInstantiationSet().getBoxISet().iterator();
        while (it.hasNext()) {
            Term in = ((Term) it.next()).copy();
            ModalityIndex r = ((Box) in).getIndex();
            is.buildISet(in.negate().box(r.copy()).box(r.copy()).box(r.copy()), this.getModalityIndex());
        }
        return is;
    }
    
    public final String getName() {
        String out = this.name;
        if (this.isModeComposition()==true) {
            out += "o";
        }
        return out;
    }
//    public static void main(String[] args) {
//        ;
//    }
}
