package jasper.element.axioms;
import jasper.element.*;
import java.util.*;
import jasper.data.*;
import jasper.element.symbols.*;
import jasper.element.operators.*;
//Axiomatic Translation, version 1.1.0, @K.J.Smith 2007-2008, http://project.kjsmith.net/
public class AxiomAlt1_22 extends Axiom {
    
    private final String name = "AxiomAlt1_22";
    
    public AxiomAlt1_22 (ModalityIndex index) {
        super(index);
        this.setModeComposition(true);
    }
    
    // SUMMARY: 	New symbols have been defined of the form: []¬p
    // Alt1^22 ¬[][][]p -> [][][]¬p 
    // Ax(Auy[¬Rxu \/ ¬Ruy \/ ¬Q¬[]rp(y)] \/ Avz[¬Rxv \/ ¬Rvz \/ Q[]¬rp(z)])
    protected final Term getResult(InstantiationSet iset) {
        And returnedTerm = null;
        SymbolCache sc = this.getTM().getSymbolCache();
        FOLVariable X = sc.getVariable(VarX);
        FOLVariable Y = sc.getVariable(VarY);
        FOLVariable Z = sc.getVariable(VarZ);
        FOLVariable U = sc.getVariable(VarU);
        FOLVariable V = sc.getVariable(VarV);
        Iterator it = iset.getBoxISet().iterator();
        
        while (it.hasNext()) {
            Term out = null;
            Box def = ((Box) it.next()).copy();
            Predicate symbolNotBoxP = def.copy().negate().getP(); //Q¬[]p
            ModalityIndex r = def.getIndex().copy();
            Predicate symbolBoxNotP = def.copy().getTerm().negate().box(r).getP(); //Q[]¬p
            
            symbolNotBoxP.setArg(Y);
            Term symbolNotBoxPTerm = symbolNotBoxP.negate(); //¬Q¬[]p(y)
            symbolBoxNotP.setArg(Z); //Q[]¬p(z)
            
            AccessibilityRelation Rxu = sc.getAccessibilityRelation(def.getIndex().copy());
            r = def.getIndex().copy();
            Rxu.setVariables(X, U, r); //Rxy
            Term RxuTerm = Rxu.negate();
            
            AccessibilityRelation Ruy = sc.getAccessibilityRelation(def.getIndex().copy());
            r = def.getIndex().copy();
            Ruy.setVariables(U, Y, r); //Rxy
            Term RuyTerm = Ruy.negate();
            
            AccessibilityRelation Rxv = sc.getAccessibilityRelation(def.getIndex().copy());
            r = def.getIndex().copy();
            Rxv.setVariables(X, V, r); //Rxy
            Term RxvTerm = Rxv.negate();
            
            AccessibilityRelation Rvz = sc.getAccessibilityRelation(def.getIndex().copy());
            r = def.getIndex().copy();
            Rvz.setVariables(V, Z, r); //Rxy
            Term RvzTerm = Rvz.negate();
            
            //  Auy[¬Rxu \/ ¬Ruy \/ ¬Q¬[]rp(y)]
            Term firstTerm = new Forall(new Or(RxuTerm, RuyTerm, symbolNotBoxPTerm), new QuantifierList(U,Y));
            //  Avz[¬Rxv \/ ¬Rvz \/ Q[]¬rp(z)]
            Term secondTerm = new Forall(new Or(RxvTerm, RvzTerm, symbolBoxNotP), new QuantifierList(V,Z));
            out = new Forall(new Or(firstTerm, secondTerm), new QuantifierList(X));
            if (returnedTerm==null) {
                returnedTerm = new And(out);
            } else {
                returnedTerm.setTerm(out);
            }
        }
        return returnedTerm;
    }
    
    public Def[] getDef() { // []¬p
        ArrayList defs = new ArrayList();
        if ((this.getBoxNotSet()==null) 
                || (this.getBoxNotSet().getBoxISet()==null)) {
            return null;
        }
        Iterator it = this.getBoxNotSet().getBoxISet().iterator();
        while (it.hasNext()) {
            defs.add(new Def(((Term) it.next()).copy()));
        }
        return (Def[]) defs.toArray(new Def[defs.size()]);
    }
    
    private InstantiationSet getBoxNotSet() {  // []¬p
        InstantiationSet is = new InstantiationSet();
        if ((this.getInstantiationSet()==null)
                ||(this.getInstantiationSet().getBoxISet()==null)) {
                return null;
            }
        Iterator it = this.getInstantiationSet().getBoxISet().iterator();
        while (it.hasNext()) {
            Term in = ((Term) it.next()).copy();
            ModalityIndex r = ((Box) in).getIndex();
            is.buildISet(((Box) in).getTerm().negate().box(r.copy()), this.getModalityIndex());
        }
        return is;
    }
    
    //axiom             new             composition
    //alt1_22           []¬p            [][][]p & [][][]¬p
    protected InstantiationSet getISet() { 
        InstantiationSet is = new InstantiationSet();
        if ((this.getInstantiationSet()==null)
                ||(this.getInstantiationSet().getBoxISet()==null)) {
                return null;
            }
        Iterator it = this.getInstantiationSet().getBoxISet().iterator();
        while (it.hasNext()) {
            Term in = ((Term) it.next()).copy();
            ModalityIndex r = ((Box) in).getIndex();
            is.buildISet(((Box) in).getTerm().negate().box(r.copy()).box(r.copy()).box(r.copy()), this.getModalityIndex());
        }
        it = this.getInstantiationSet().getBoxISet().iterator();
        while (it.hasNext()) {
            Term in = ((Term) it.next()).copy();
            ModalityIndex r = ((Box) in).getIndex();
            is.buildISet(((Box) in).box(r.copy()).box(r.copy()), this.getModalityIndex());
        }
        return is;
    }
    
    public final String getName() {
        String out = this.name;
        if (this.isModeComposition()==true) {
            out += "o";
        }
        return out;
    }
//    public static void main(String[] args) {
//        ;
//    }
}
