package jasper.element.axioms;
import jasper.element.symbols.*;
import jasper.element.*;
import java.util.*;
import jasper.data.*;
import jasper.element.operators.*;
//Axiomatic Translation, version 1.1.0, @K.J.Smith 2007-2008, http://project.kjsmith.net/
public class AxiomW extends Axiom {
    
    private final String name = "AxiomW";

    public AxiomW(ModalityIndex index) {
        super(index);
        this.setModeComposition(true);
        this.setModeCf5(true);
    }

    //Ax(Ey(R(x,y)  /\  Q[]p(y)  /\  ¬Qp(y))  \/  Q[]p(x))
    protected final Term getResult(InstantiationSet iset) {
        And returnedTerm = null;
        SymbolCache sc = this.getTM().getSymbolCache();
        FOLVariable X = sc.getVariable(VarX);
        FOLVariable Y = sc.getVariable(VarY);
        Iterator it = iset.getBoxISet().iterator();
        
        while (it.hasNext()) {
            Term out = null;
            Box def = ((Box) it.next()).copy();
            Predicate symbolBoxP = def.copy().getP(); //Q[]p
            Predicate symbolP = def.getTerm().copy().getP(); //Qp
            
            AccessibilityRelation Rxy = sc.getAccessibilityRelation(def.getIndex().copy());
            ModalityIndex r = def.getIndex().copy();
            Rxy.setVariables(X, Y, r); //Rxy
            
            Predicate first = symbolBoxP.copy();
            first.setArg(Y); 
            
            Predicate second = symbolP.copy();
            second.setArg(Y);
            Term secondTerm = second.negate();
            
            Predicate third = symbolBoxP.copy();
            third.setArg(X);
            
            out = new Exists(new And(Rxy, first, secondTerm), new QuantifierList(Y));
            out = new Forall(new Or(out, third), new QuantifierList(X));
            if (returnedTerm==null) {
                returnedTerm = new And(out);
            } else {
                returnedTerm.setTerm(out);
            }
        }
        return returnedTerm;
    }

    public Def[] getDef() { 
        if (this.isModeCf5()==true) {
            ArrayList defs = new ArrayList();
            if ((this.getISet()==null) 
                || (this.getISet().getBoxISet()==null)) {
                return null;
            }
            Iterator it = this.getISet().getBoxISet().iterator();
            while (it.hasNext()) {
                defs.add(new Def(((Term) it.next()).copy()));
            }
            return (Def[]) defs.toArray(new Def[defs.size()]);
        } else { //no new def terms
            return null;
        }
    }

    protected InstantiationSet getISet() {  //[]([]p->p) .... []¬([]p/\¬p) 
        InstantiationSet is = new InstantiationSet();
        if ((this.getInstantiationSet() == null) 
                || (this.getInstantiationSet().getBoxISet() == null)) {
            return null;
        }
        Iterator it = this.getInstantiationSet().getBoxISet().iterator();
        while (it.hasNext()) {
            Box in = ((Box) it.next()).copy();
            ModalityIndex r = in.getIndex();
            is.buildISet((new And(in.getTerm().copy().negate(), in.copy())
                    ).negate().box(r.copy()), this.getModalityIndex());
        }
        return is;
    }
    
    public final String getName() {
        String out = this.name;
        if (this.isModeComposition()==true) {
            out += "o";
        }
        return out;
    }
//    public static void main(String[] args) {
//        ;
//    }
}
