package jasper.element.operators;
import jasper.element.symbols.*;
import jasper.element.interfaceAbstract.*;
import jasper.element.*;
import jasper.exceptions.*;
import java.util.*;
//Axiomatic Translation, version 1.1.0, @K.J.Smith 2007-2008, http://project.kjsmith.net/
public class QuantifierList extends Term implements TermElement {

    private final ArrayList termList = new ArrayList();
    private final static String SymbolName = "";
     
    public QuantifierList(FOLVariable variable)
            throws TermCompositionException {
        super(QuantifierList.SymbolName);
        this.setTerm(variable);
    }

    public QuantifierList(FOLVariable v1, FOLVariable v2)
            throws TermCompositionException {
        super(QuantifierList.SymbolName);
        this.setTerm(v1);
        this.setTerm(v2);
    }
    
    public QuantifierList(FOLVariable v1, FOLVariable v2, FOLVariable v3)
            throws TermCompositionException {
        super(QuantifierList.SymbolName);
        this.setTerm(v1);
        this.setTerm(v2);
        this.setTerm(v3);
    }
    
    public QuantifierList(FOLVariable v1, FOLVariable v2, FOLVariable v3, FOLVariable v4)
            throws TermCompositionException {
        super(QuantifierList.SymbolName);
        this.setTerm(v1);
        this.setTerm(v2);
        this.setTerm(v3);
        this.setTerm(v4);
    }
    
    public QuantifierList(ArrayList list)
            throws TermCompositionException {
        super(QuantifierList.SymbolName);
        this.setTerm(list);
    }

    public void setTerm(FOLVariable variable)
            throws TermCompositionException {
        if (variable==null) { 
            throw new JasperRuntimeException();
        }
        this.termList.add(variable);
    }

    public void setTerm(ArrayList list)
            throws TermCompositionException {
        if (list==null) { 
            throw new JasperRuntimeException();
        }
        if (list.isEmpty() == true) {
            throw new TermCompositionException();
        }
        Iterator i = list.iterator();
        while (i.hasNext()) {
            Term tmp = ((Term) i.next()).copy();
            if ( (tmp==null) 
                || ((tmp instanceof FOLVariable)==false) ) {
                throw new TermCompositionException();
            }
        }
        this.termList.addAll(list);
    }

    public boolean equals (Term term) {
        if (term==null) {
            throw new JasperRuntimeException();
        }
        if ((term instanceof QuantifierList)==false) {
            return false;
        }
        ArrayList l = ((QuantifierList)term).getList();
        if (l.size()!=this.termList.size()) {
            return false;
        }
        
        Iterator it = this.termList.iterator();
        while (it.hasNext()) {
            FOLVariable v = (FOLVariable) it.next();
            Iterator it2 = l.iterator();
            boolean found = false;
            while (it2.hasNext()) {
                FOLVariable v2 = (FOLVariable) it2.next();
                if (v.equals(v2) == true) {
                    found = true;
                }
            }
            if (found == false) {
                return false;
            }
        }
        //now other way around
        it = l.iterator();
        while (it.hasNext()) {
            FOLVariable v = (FOLVariable) it.next();
            Iterator it2 = this.termList.iterator();
            boolean found = false;
            while (it2.hasNext()) {
                FOLVariable v2 = (FOLVariable) it2.next();
                if (v.equals(v2) == true) {
                    found = true;
                }
            }
            if (found == false) {
                return false;
            }
        }
        
        return true;
    }

    public ArrayList getList() {
        ArrayList out = new ArrayList();
        Iterator it = this.termList.iterator();
        while (it.hasNext()) {
            out.add(((Term)it.next()).copy());
        }
        return out;
    }
 
    public String toString() {
        Iterator it = this.termList.iterator();
        String out = "";
        while (it.hasNext()) {
            out += ((FOLVariable)it.next()).toString();
            if (it.hasNext()) {
                out += ",";
            }
        }
        return out;
    }

    public String inString() {
        return this.toString();
    }
    
    public String asString() {
        return this.toString();
    }
    
    public QuantifierList copy() {
        ArrayList outList = new ArrayList();
        Iterator it = this.termList.iterator();
        while (it.hasNext()) {
            outList.add(((FOLVariable)it.next()).copy());
        }
        return new QuantifierList(outList);
    }

    public ArrayList descendList() {
        throw new TermCompositionException();
    }
    
//    public static void main(String[] args) {
//        ;
//    }
}
